@extends('layouts.master')

@section('navigation')
  @include('local.managers.admins._navigation')
@stop

@section('main')

<div class="row">
  <div class="col-md-12">
    <div class="btn-group pull-right">
      {{ HTML::linkAction('TemplatesController@printview', 'Print', array($template->id), array('class'=>'btn btn-default')) }}
    </div>

    <p>Passing Criteria:
      <span id="expected_percentage">{{{ $template->expected_percentage }}}% of students must obtain at least </span>
      <span id="expected_points">{{{$template->expected_points}}}</span> points
    </p>

    <table class="table table-striped table-condensed datatable">
      <thead>
          <tr>
            <th></th>
            <th>Criterion</th>
            <th>Beginning (1-2)</th>
            <th>In Progress (3-4)</th>
            <th>Good (5-6)</th>
            <th>Excellent (7-8)</th>
            <th>Learning Outcome</th>
          </tr>
        </thead>
      <tbody>
      @foreach(json_decode($template->contents) as $index => $criterion)

        <tr data-criterion-copyright="{{ $criterion->copyright }}" data-criterion-notes="{{ $criterion->notes }}">
          @if(isset($criterion->notes))
            <td>{{ $index + 1 }}.</td>
            <td>
              <span><em data-toggle="tooltip" data-placement="top" title="{{ $criterion->notes }}">{{{ $criterion->name }}}</em></span>
              <sup></sup>
              @if(property_exists($criterion, 'subcriteria'))
                <ul class="list-unstyled">
                  @foreach($criterion->subcriteria as $subcriterion)
                      <li>{{ $subcriterion }}</li>
                  @endforeach
                </ul>
              @endif
            </td>
          @else
            <td>{{ $index + 1 }}.</td>
            <td>
              <span>{{{ $criterion->name }}}</span>
              <sup></sup>
              @if(property_exists($criterion, 'subcriteria'))
                <ul class="list-unstyled">
                  @foreach($criterion->subcriteria as $subcriterion)
                      <li>{{ $subcriterion }}</li>
                  @endforeach
                </ul>
              @endif
            </td>
          @endif

          <td>{{{ $criterion->description12 }}}</td>
          <td>{{{ $criterion->description34 }}}</td>
          <td>{{{ $criterion->description56 }}}</td>
          <td>{{{ $criterion->description78 }}}</td>
          <td>{{{ Outcome::where('id', $criterion->outcome_id)->first()->name }}}</td>

        </tr>
      @endforeach
      </tbody>
    </table>


    <br>
    <br>
    <div id="copyright-info">
      <hr>
      <p class="small"><strong>Copyright Information</strong></p>
      <ul id="copyright-list" class="list-unstyled small">
      </ul>
      <hr>
    </div>


    <br>
    <br>
    <br>

  </div>
</div>
@stop

@section('included-js')
    @include('global._datatables_js')
@stop

@section('javascript')

// --------------------------------------------------------------------------
// Page load
// --------------------------------------------------------------------------

buildCopyrightList();

// --------------------------------------------------------------------------
// Functions
// --------------------------------------------------------------------------

// Build list from copyright info in template
function buildCopyrightList()
{
  // Empty the copyright list
  $('#copyright-list').empty();

  $('tbody tr').each(function( index )
  {
    var criterion = $(this);
    // If there's copyright info (in this particular view, it checks for empty strings)
    if(criterion.data('criterion-copyright')!='')
    {
      var copyright = criterion.data('criterion-copyright');

      // If there is anything in the copyright list
      if($('#copyright-list li').length>0)
      {

        // Check copyright list for the same copyright text
        var found = false;
        $('#copyright-list li').each(function()
        {
          // If found, give the string its number
          if(copyright==$(this).find('span').text())
          {
            copyrightNumber = Number.parseInt($(this).find('sup').text());


            criterion.children('td:nth-child(2)').find('sup').text(copyrightNumber);
            found =true;

            //to break
            return false;
          }
        });

        // Otherwise, give it the next number and append a new item to the
        // list
        if(!found)
        {
          var copyrightNumber = $('#copyright-list li').length+1;
          criterion.children('td:nth-child(2)').find('sup').text(copyrightNumber);
          $('#copyright-list').append('<li><sup>'+copyrightNumber+' </sup><span>'+copyright+'<span></li>');
        }
      }
      // Otherwise, give it number 1 and append it
      else
      {

        criterion.children('td:nth-child(2)').find('sup').text('1');
        $('#copyright-list').append('<li><sup>1 </sup><span>'+copyright+'<span></li>');
      }
    }
  });

  if($('#copyright-info li').length>0)
  {
    $('#copyright-info').show();
  }
  else
  {
    $('#copyright-info').hide();
  }
}


@stop
